Package: interval
Type: Package
Title: Weighted Logrank Tests and NPMLE for Interval Censored Data
Version: 1.1-1.0
Date: 2023-08-24
Author: Michael P. Fay
Maintainer: Michael P. Fay <mfay@niaid.nih.gov>
Depends: R (>= 2.2.1), stats, survival, perm (>= 1.0), Icens, MLEcens
Suggests: coin
LazyLoad: yes
Description: Functions to fit nonparametric survival curves, plot them, and perform logrank or Wilcoxon type tests [see Fay and Shaw <doi:10.18637/jss.v036.i02>].
License: GPL (>= 2)
NeedsCompilation: no
Packaged: 2023-08-24 20:14:24 UTC; faym
Repository: CRAN
Date/Publication: 2023-08-24 20:40:02 UTC
Built: R 4.4.0; ; 2024-05-03 11:10:14 UTC; unix
